# --------------------------------------------------
# Météo (Version 1.3)
# By Sha-Chan~
# June 2020
#
# Code provided with licence (CC BY-NC-SA 4.0)
# For more information about licence :
# https://creativecommons.org/licenses/by-nc-sa/4.0/
# --------------------------------------------------

from math import pow, floor

class Param:
    def __init__(self, evo = 0, value = 0):
        self.evo = evo
        self.value = value

class System:
    def __init__(self, t_val, t_evo, hr_val, hr_evo, pr_val, pr_evo, wind_val, wind_evo, alt, std_pr, dew, cloud):
        self.t = Param(t_evo, t_val)      
        self.hr = Param(hr_evo, hr_val)
        self.pr = Param(pr_evo, pr_val)
        self.wind = Param(wind_evo, wind_val)
        self.dew = dew
        self.std_pr = std_pr
        self.alt = alt
        self.cloud = cloud
        
        self.r_type = set()
    
    def rain(self):
        self.r_type = set()
        
        if "St" in self.cloud: self.r_type.add("de bruine")
        for i in ("As", "Ns", "Cb", "Sc", "Ac"):
            if i in self.cloud: self.r_type.add(("de pluie", "de neige")[self.t.value < 0])
        if "Cb" in self.cloud or "Ns" in self.cloud:
            self.r_type.add("d'averses")
            if "Cb" in self.cloud:
                self.r_type.add("de grêle")
                self.r_type.add("d'orage")

    def rain_probability(self):        
        i = 0
        if self.t.value - self.dew < 6:
            if self.t.value - self.dew <= 2: i += 3
            else: i += 2
        
        delta = self.std_pr - self.pr.value
        if delta > 0 or self.wind.value == 1:
            if delta > 10: i += 3
            elif delta > 5: i += 2
            else: i += 1
        else: i -= 1
        
        if "Cb" in self.cloud or "Nb" in self.cloud: i += 1
        
        if i > 5: i = 5
        elif i < 0: i = 0
        
        return ("très peu probable", "peu probable", "assez probable", "probable", "très probable", "imminent")[i]

    def output(self):
        self.rain()
        
        txt = "--- Données ---\nTempérature : {0} °C\nHygrométrie : {1} %\nPoint de rosée : {2} °C\nPression : {3} hPa\nPression std : {4} hPa\n\n--- Court terme ---\n".format(floor(self.t.value), floor(self.hr.value), floor(self.dew), floor(self.pr.value), floor(self.std_pr))

        cl_name = {"Cu":"Cumulus",
                   "As":"Altocumulus",
                   "Cc":"Cirrocumulus",
                   "Cb":"Cumulonimbus",
                   "Sc":"Stratocumulus",
                   "St":"Stratus",
                   "As":"Altostratus",
                   "Cs":"Cirrostratus",
                   "Ns":"Nimbostratus",
                   "Ci":"Cirrus"}
        
        cloud = [cl_name[i] for i in cl_name.keys() if i in self.cloud]
        
        if len(self.cloud):
            txt += "Nuages identifés : " + ", ".join(cloud) + ". "
            if len(self.r_type): txt += "Il y a un risque " + ", ".join(self.r_type) + ". Le risque de précipitation est estimé {}. ".format(self.rain_probability())
            else: txt += "Pas de risque de précipitation, estimation jugée probable. "
        else: txt += "Les nuages n'ont pas été identifiés. "

        if self.wind.evo == 2 and self.pr.evo == 2 and self.t.evo == 2 and self.hr.evo == 2: txt += "Le temps est stable. "
        elif self.wind.evo != 3 and self.wind.value == 2:
            if self.pr.evo == 1: txt += "Le ciel va se dégager. "
            elif self.pr.evo == 3: txt += "La couverture nuageuse va s'intensifier. "

        if self.wind.evo == 2 and self.wind.value == 3: txt += "Un changement rapide de pression n'est pas à attendre. "
            
        if self.t.evo == 1:
            if self.hr.evo in (2, 3): txt += "Des éclaircies sont à venir. "
        elif self.t.evo == 2:
            if self.hr.evo == 1: txt += "Une chute du plafond est à attendre, ainsi qu'un risque de brume. "
            elif self.hr.evo == 3: "Des éclaircies sont à venir. "
        elif self.t.evo == 3:
            if self.hr.evo in (1, 2): txt += "Une chute du plafond est à attendre, ainsi qu'un risque de brume. "

        if self.t.value - self.dew < 5 and self.wind.value != 1:
          txt += "Formation de brume estimée {}. ".format(("imminente", "très probable", "probable", "assez probable", "peu probable")[floor(self.t.value - self.dew)])
          if self.t.evo == 1 or self.hr.evo == 3: txt += "Le risque d'apparition de brume va chuter. "
          elif self.t.evo == 2 or self.hr.evo == 1: txt += "Le risque d'apparition de brume va augmenter. "
          
        if "Cu" in self.cloud and self.t.value - self.dew >= 5: txt += "La base des cumulus est estimée à {} m.".format(_alt_base(self.t.value, self.dew))


        txt += "\n\n--- Moyen terme ---\n"

        if "Sc" in self.cloud: txt += "Une rechute est encore possible, mais les conditions générales vont s'améliorer."
        elif "Cb" in self.cloud: txt += "Le front froid est arrivé, il y a un risque d'orage."
        elif "St" in self.cloud: txt += "Une rechute du temps reste possible avec l'arrivée d'un front froid."
        elif "Ns" in self.cloud: txt += "Le front chaud est sur vous, une légère amélioration est à attendre avec l'apparition de nuages plus petits : Cumulus, Stratus en général dans un délai de quelques heures."
        elif "As" in self.cloud: txt += "Quelques pluies peuvent arriver dans les heures à venir. Une dégradation plus franche du temps est à attendre."               
        elif "Cc" in self.cloud or "Cs" in self.cloud: txt += "Un épisode pluvieux arrivera d'ici quelques jours."
        elif "Ci" in self.cloud: txt += "Le temps va se dégrader, une perturbation se dirige vers vous."
        elif "Cu" in self.cloud: txt += "Le temps devrait rester au beau pendant quelques jours."
        else: txt += "Prévisions indisponibles."
        return print(txt)


_dew = lambda t, hr: pow(hr/100, 1/8) * (112 + 0.9*t) + (0.1*t) - 112

_std_pression = lambda alt: 1013.25 - (alt / (28/3))

_alt_base = lambda t, dew: floor(125 * (t - dew))

def _enter():
    args = [0 for i in range(12)]

    args[0] = float(input("Température (°C) :\n> "))
    while args[1] not in (1, 2, 3):
        args[1] = int(input("Evolution de la température :\n1 - en augmentation\n2 - stable\n3 - en chute\n> "))

    args[2] = float(input("\nHygrométrie (%) :\n> "))
    while args[3] not in (1, 2, 3):
        args[3] = int(input("Evolution de l'hygrométrie :\n1 - en augmentation\n2 - stable\n3 - en chute\n> "))

    args[4] = float(input("\nPression (hPa) :\n> "))
    while args[5] not in (1, 2, 3):
        args[5] = int(input("Evolution de la pression :\n1 - en augmentation\n2 - stable\n3 - en chute\n> "))

    while args[6] not in (1, 2, 3):
        args[6] = int(input("\nForce du vent :\n1 - rafaleux, violent\n2 - régulier\n3 - brise, léger\n> "))
    while args[7] not in (1, 2, 3):
        args[7] = int(input("Evolution du vent :\n1 - en augmentation\n2 - stable\n3 - en chute\n> "))

    args[8] = int(input("\nAltitude (m) :\n> "))
    args[9] = _std_pression(args[8])
    args[10] = _dew(args[0], args[2])
    
    tx, cl, hs, al = 0, 0, 0, 0
    args[11] = set()
        
    while tx not in (1, 2):
        tx = int(input("\nAspect des nuages :\n1 - cumuliformes\n2 - stratiformes\n> "))
    while cl not in (1, 2, 3):
        cl = int(input("Couleur des nuages \n1 - gris foncé\n2 - gris nuancé\n3 - gris clair, blanc\n> "))
    while al not in (1, 2):
      al = int(input("Altitude des nuages :\n1 - haut\n2 - bas\n> "))
    while hs not in (1, 2 , 3):
        hs = int(input("Occlusion du ciel :\n1 - occlusion totale\n2 - occlusion partielle\n3 - faible occlusion\n> "))

    if al == 2 and cl in (1, 2) and hs in (1, 2): args[11].add("Cb")
    if (al, tx, cl) == (1, 1, 3) and hs in (2, 3): args[11].add("Cc")
    if (al, tx, cl, hs) == (1, 1, 3, 2): args[11].add("Ac")
    if (al, tx, cl) == (2, 1, 3) and hs in (2, 3): args[11].add("Cu")

    if (al, cl) == (2, 2) and hs in (1, 2): args[11].add("Nb")
    if (al, tx, cl, hs) == (1, 2, 3, 3): args[11].add("Cs")
    if (al, tx, cl) == (1, 2, 1) and hs in (1, 2): args[11].add("As")
    if (al, tx) == (2, 2) and cl in (2, 3) and hs in (2, 3): args[11].add("St")

    if (al, tx, cl, hs) == (2, 1, 2, 2): args[11].add("Sc")
    if (al, tx, cl, hs) == (1, 2, 3, 3): args[11].add("Ci")
    
    return args
    
meteo = lambda: System(*_enter()).output()
